#include "MESH_ConnectedComComputerCoreVisitor.h"
#include "GraphTester.h"
#include <iostream>

using namespace std;

namespace X3DTK {
namespace MESH {

ConnectedComComputerCoreVisitor::ConnectedComComputerCoreVisitor(Progress *progress)
: CoreVisitor()
{
  // Enter and leave functions.
  define(Recorder<Vertex>::getEnterFunction(&ConnectedComComputerCoreVisitor::enterVertex));
  define(Recorder<Shape>::getWalkOnFunction(&ConnectedComComputerCoreVisitor::walkOnShape));
  
  Singleton<ConnectedComComputerStateVariables>::getInstance()->setProgress(progress);
}

bool ConnectedComComputerCoreVisitor::walkOnShape(Shape *, SFNode child) 
{
  ConnectedComComputerStateVariables *stateVariables = Singleton<ConnectedComComputerStateVariables>::getInstance();
  MFNode selectedNodes = stateVariables->getSelectedNodes();
  return (selectedNodes.empty() || (std::find(selectedNodes.begin(), selectedNodes.end(), child) != selectedNodes.end()));
}

void ConnectedComComputerCoreVisitor::enterVertex(Vertex *V) 
{
  ConnectedComComputerStateVariables *stateVariables = Singleton<ConnectedComComputerStateVariables>::getInstance();
  
  if (!stateVariables->getNode(V))
  {
    // computing the list of Mesh parents of V.
    MFNode parentList = V->getParentList();
    std::list<Mesh *> meshList;
    for (MFNode::const_iterator p = parentList.begin(); p != parentList.end(); ++p)
    {
      if (dynamic_cast<Mesh *>(*p) != 0)
        meshList.push_back(static_cast<Mesh *>(*p));
    }
    
    // defining structures.
    std::vector<SFFace *> stack;
    unsigned int stackSize = 0;
    std::list<Mesh *>::const_iterator m = meshList.begin();
    while (m != meshList.end())
    {
      stackSize += (*m)->getFaces().size();
      ++m;
    }  
    stack.reserve(stackSize);
    stateVariables->getProgress()->setTotalSteps(stackSize);
    unsigned int step = 0;
    
    std::list<Mesh *>::const_iterator itMesh = meshList.begin();
    if (itMesh == meshList.end()) return;
    
    Mesh::MFFace::const_iterator itFace = (*itMesh)->getFaces().begin();
    
    while (itMesh != meshList.end())
    {
      std::list<Mesh *> cmeshList;
      std::map<SFFace *, Mesh *> faceMap;
      stack.push_back(*itFace);
      stateVariables->getProgress()->setProgress(++step);

      std::vector<SFFace *>::const_iterator f = stack.end();
      --f;
      faceMap[*f] = *itMesh;
      if (std::find(cmeshList.begin(), cmeshList.end(), *itMesh) == cmeshList.end())
        cmeshList.push_back(*itMesh);
      
      while (f != stack.end())
      {
        const SFFace::MFEdge &me = (*f)->getEdges();
        for (SFFace::MFEdge::const_iterator e = me.begin(); e != me.end(); ++e)
        { 
          const SFEdge::MFFace &mlf = (*e)->getLeftFaces();
          const SFEdge::MFFace &mrf = (*e)->getRightFaces();
        
          for (SFEdge::MFFace::const_iterator lf = mlf.begin(); lf != mlf.end(); ++lf)
          {
            if (faceMap.find(*lf) == faceMap.end())
            {
              // finding the Mesh                            
              for (std::list<Mesh *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
              {
                const Mesh::MFFace &faces = (*it)->getFaces();
                if (faces.find(*lf) != faces.end())
                {
                  faceMap[*lf] = *it;
                    
                  if (std::find(cmeshList.begin(), cmeshList.end(), *it) == cmeshList.end())
                    cmeshList.push_back(*it);

                  break;    
                }
              }  
              
              stack.push_back(*lf);
              stateVariables->getProgress()->setProgress(++step);
            }
          }
        
          for (SFEdge::MFFace::const_iterator rf = mrf.begin(); rf != mrf.end(); ++rf)
          {
            if (faceMap.find(*rf) == faceMap.end())
            {
              // finding the Mesh                            
              for (std::list<Mesh *>::const_iterator it = meshList.begin(); it != meshList.end(); ++it)
              {
                const Mesh::MFFace &faces = (*it)->getFaces();
                if (faces.find(*rf) != faces.end())
                {
                  faceMap[*rf] = *it;
                    
                  if (std::find(cmeshList.begin(), cmeshList.end(), *it) == cmeshList.end())
                    cmeshList.push_back(*it);

                  break;    
                }
              }  

              stack.push_back(*rf);
              stateVariables->getProgress()->setProgress(++step);
            }
          }
        }
        ++f;
      }
      
      // creating new Meshs and Vertex.
      std::map<Mesh *, Mesh *> meshMap;
      Vertex *newVertexNode = new Vertex();
      for (std::list<Mesh *>::const_iterator it = cmeshList.begin(); it != cmeshList.end(); ++it)
      {
        Mesh *newMesh = new Mesh();
        newMesh->setVertex(newVertexNode);
        stateVariables->addNode(newVertexNode);
        
        // filling Mesh datas.
        newMesh->data().setNormal((*it)->data().hasNormal());
        newMesh->data().setNormalPerVertex((*it)->data().getNormalPerVertex());
        newMesh->data().setCreaseAngle((*it)->data().getCreaseAngle());

        newMesh->data().setColor((*it)->data().hasColor());
        newMesh->data().setColorPerVertex((*it)->data().getColorPerVertex());
        newMesh->data().setRGBA((*it)->data().getRGBA());

        newMesh->data().setTexCoord((*it)->data().hasTexCoord());
        
        newMesh->data().setSolid((*it)->data().isSolid());

        meshMap[*it] = newMesh;
      }
      
      // iterating the faces of the connected component.
      std::map<unsigned int, unsigned int> newIndex;
      
      for (std::map<SFFace *, Mesh *>::const_iterator fm = faceMap.begin(); fm != faceMap.end(); ++fm)
      {
        MFInt32 faceCoordIndex;
        SFFace *face = (*fm).first;
 
        const SFFace::MFEdge &me = face->getEdges();
        SFFace::MFEdge::const_iterator e = me.begin();
        
        while (e != me.end())
        {
          SFVertex *v = (*e)->getFromVertex();
          if (newIndex.find(v->getIndex()) == newIndex.end())
          {
            SFVertex *newVertex = newVertexNode->createVertex();
            newVertex->data().setPoint(v->data().getPoint());
            newIndex[v->getIndex()] = newVertex->getIndex(); 
          }
          ++e;
          faceCoordIndex.push_back(newIndex[v->getIndex()]);
        }
        
        // filling face data.
        SFFace *newFace = meshMap[(*fm).second]->createFace(faceCoordIndex);

        newFace->data().setNormal(face->data().getNormal());
        newFace->data().setColor(face->data().getColor());

        // filling vertex data.
        const SFFace::MFEdge &nme = newFace->getEdges();
        e = me.begin();
        SFFace::MFEdge::const_iterator ne = nme.begin();
        
        while (e != me.end())
        {
          SFVertex *v = (*e)->getFromVertex();
          SFVertex *newVertex = (*ne)->getFromVertex();
          
          newVertex->data().setNormalOfFace(v->data().getNormalOfFace(face), newFace);
          newVertex->data().setColorOfFace(v->data().getColorOfFace(face), newFace); 
          newVertex->data().setTexCoordOfFace(v->data().getTexCoordOfFace(face), newFace);  
          
          ++e;
          ++ne;  
        }
      }
      
      // adding new Meshes to the scene graph.
      for (std::map<Mesh *, Mesh *>::const_iterator mm = meshMap.begin(); mm != meshMap.end(); ++mm)
      {
        Mesh *mesh = (*mm).first;
        Mesh *newMesh = (*mm).second;
        
        MFNode parentList = mesh->getParentList();
        for (MFNode::const_iterator p = parentList.begin(); p != parentList.end(); ++p)
        {
          SFNode cp = (*p)->clone();
          cp->setChild(newMesh);
          
          MFNode gparentList = (*p)->getParentList();
          
          for (MFNode::const_iterator gp = gparentList.begin(); gp != gparentList.end(); ++gp)
            (*gp)->setChild(cp);
        }
      }
      
      // finding the next face.
      while ((itMesh != meshList.end()) && (std::find(stack.begin(), stack.end(), *itFace) != stack.end()))
      {
        ++itFace;
        if (itFace == (*itMesh)->getFaces().end())
        {
          ++itMesh;
          if (itMesh != meshList.end())
            itFace = (*itMesh)->getFaces().begin();
        }    
      }
    }

    // remove old meshes.
    m = meshList.begin();
    while (m != meshList.end())
    {
      MFNode parentList = (*m)->getParentList();
      for (MFNode::const_iterator p = parentList.begin(); p != parentList.end(); ++p)
      {
        MFNode gparentList = (*p)->getParentList();
        for (MFNode::const_iterator gp = gparentList.begin(); gp != gparentList.end(); ++gp)
          (*gp)->removeChild(*p);

        stateVariables->addToDeleteNode(*p);
        ++m;
      }
    }
    stateVariables->addNode(V);
  }
}

}
}  
