#include "X3D_SceneSaver.h"
#include "DFSGraphTraversal.h"
#include "X3D_SceneSaverGetCoreVisitor.h"
#include "X3D_SceneSaverSetCoreVisitor.h"
#include "X3D_SceneSaverWriteCoreVisitor.h"
#include "X3D_Scene.h"

#include <fstream>
#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

SceneSaver::SceneSaver()
: X3DMultiPassProcessor()
{
  setGraphTraversal("Get", new DFSGraphTraversal());  
  setComponentVisitor("Get", new SceneSaverGetCoreVisitor());

  setGraphTraversal("Set", new DFSGraphTraversal());  
  setComponentVisitor("Set", new SceneSaverSetCoreVisitor());
  
  setGraphTraversal("Write", new DFSGraphTraversal());  
  setComponentVisitor("Write", new SceneSaverWriteCoreVisitor());
}

SceneSaver::~SceneSaver()
{
  Singleton<SceneSaverStateVariables>::removeInstance();
}

void SceneSaver::saveAs(Scene *S, const SFString &fileName)
{
  if (S == 0)
  {
    cx3d << "unable to write to file, because the Scene is NULL!" << endl;
    return;
  }
    
  Singleton<SceneSaverStateVariables>::getInstance()->init();
  traverse("Get", S);
  traverse("Set", S);
  Singleton<SceneSaverStateVariables>::getInstance()->initNodes();
  
  ofstream output(fileName);
  
  output << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
  output << "<X3D>" << endl;
  output << "  <head>" << endl; 
  std::list<std::pair<SFString, SFString> > history = S->getProcessHistory().getProcessMap();
  for (std::list<std::pair<SFString, SFString> >::const_iterator h = history.begin(); h != history.end(); ++h)  
    output << "    <meta name=\"process\" content=\"" + (*h).first + "\" date=\"" + (*h).second + "\"/>" << endl;
  output << "  </head>" << endl; 
  
  traverse("Write", S);
  MFString text = Singleton<SceneSaverStateVariables>::getInstance()->getOutput();
  
  for (MFString::const_iterator l = text.begin(); l != text.end(); ++l)
    output << (*l) << endl;

  output << "</X3D>" << endl;
  Singleton<SceneSaverStateVariables>::getInstance()->finish();    
}

void SceneSaver::save(Scene *S)
{
  saveAs(S, S->getFileName());
}

}
}
