#include "SFType.h"
#include "Visitor.h"

#include <algorithm>
#include <iostream>

using namespace X3DTK;
using namespace std;

int SFType::count = 0;
MFType SFType::_typeList = MFType();  
list<Visitor *> SFType::_actorList = list<Visitor *>();
bool SFType::_functionDefinition = false;

void SFType::setParent(SFType *parent)
{
  _parent = parent;
}

void SFType::addOneReference()
{
  ++_refCount;
}

void SFType::addOneReferenceRecursive()
{
  ++_refCount;
  if (_parent != 0)
    _parent->addOneReferenceRecursive();
}

void SFType::removeOneReference(SFType *type)
{
  --type->_refCount;
  
  //removes the instance of the parents.
  if (type->_parent != 0)
  {
    if (type->_refCount == 0)  
      type->_parent->_childList.erase(type->getEncodedName());

    removeOneReference(type->_parent);
  }  
  
  if (type->_refCount == 0)  
  {
    //remove in the list.      
    _typeList.erase(type->getEncodedName());
    delete type;
  }
}

SFType *SFType::getTypeOfName(const SFString &sceneGraph, const SFString &component, const SFString &name)
{
  MFType::const_iterator it = _typeList.find(encodeKey(sceneGraph, component, name)); 
  if (it != _typeList.end())
    return (*it).second;
    
  //the SFType doesn't exist.
  return 0;  
}

SFType *SFType::getTypeOfId(int id)
{
  // iterate the SFTypes
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
    if ((*it).second->getId() == id)
      return (*it).second;
      
  return 0;      
}
void SFType::beginFunctionDefinition()
{
  _functionDefinition = true;
}

void SFType::endFunctionDefinition()
{
  _functionDefinition = false;
}

SFString SFType::encodeKey(const SFString &sceneGraph, const SFString &component, const SFString &SFType)
{
  return sceneGraph + "_" + component + "_" + SFType;
}

void SFType::recomputeIds()
{
}

void SFType::printInheritanceTree()
{
  cout << "Inheritance tree:" << endl;
  SFString tab("");
  //finding the nodes which have no parent.
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
  {
    if ((*it).second->_parent == 0)
    {
      (*it).second->printInheritanceTree(tab);
    } 
  }    
  cout << "Inheritance tree finished" << endl;
}

SFType::SFType(const SFString &name, const SFString &component, const SFString &sceneGraph, X3DNodeCloner *cloner)
: _name(name), _refCount(0), _cloner(cloner)
{
  //component update.
  _component = SFComponent::getComponent(sceneGraph, component);
  if (_component == 0)
    _component = new SFComponent(component, sceneGraph);

  //component referenced by a new SFType.
  _component->addOneReference();
}

SFType::~SFType()
{
  SFComponent::removeOneReference(_component);
  _childList.clear();
  
  delete _cloner;
  
  // deleting the map
  for (map<SFString, FieldManager>::iterator a = _attributesMap.begin(); a != _attributesMap.end(); ++a)
  {
    delete (*a).second.memberFunctor;
    delete (*a).second.loader;
    delete (*a).second.writer;
  }
  
  _attributesMap.clear();
  
  for (vector<pair<SFString, X3DSFNodeFunctor *> >::iterator s = _nodeMap.begin(); s != _nodeMap.end(); ++s)
  {
    delete (*s).second;  
  }
  
  _nodeMap.clear();
  
  for (vector<pair<SFString, X3DMFNodeFunctor *> >::iterator m = _nodesMap.begin(); m != _nodesMap.end(); ++m)
  {
    delete (*m).second;  
  }
  
  _nodesMap.clear();
}

void SFType::defineTypeName(SFType **type, const SFString &name, const SFString &component, const SFString &sceneGraph, X3DNodeCloner *cloner)
{
  // Getting the name of the component.
  SFString tcomponent = component;
  if (tcomponent == "")
    tcomponent = (*type)->_component->getName();
  
  SFString tsceneGraph = sceneGraph;
  if ((tsceneGraph == "") && (*type != 0))
    tsceneGraph = (*type)->_component->_sceneGraph->getName();
     
  SFType *t = getTypeOfName(tsceneGraph, tcomponent, name);
  
  if (t == 0)
  {
    t = new SFType(name, tcomponent, tsceneGraph, cloner);
    t->_id = count;
    t->_parent = *type;  

    //adding new name.
    SFString nodeName = encodeKey(tsceneGraph, tcomponent, name);
       
    if (*type != 0)    
      (*type)->_childList[nodeName] = t;
      
      _typeList[nodeName] = t;

    if (!_functionDefinition)
    {
      ++count;
      //advertising actors
      for (list<Visitor *>::iterator itActor = _actorList.begin(); itActor != _actorList.end(); ++itActor) 
        (*itActor)->addType(t);
    }
  }
 
  //the SFType t is referenced by a new Node.
  t->addOneReference();
  *type = t;
}

X3DMemberFunctor *SFType::getMemberFunctor(const SFString &name) const
{
  map<SFString, FieldManager>::const_iterator it = _attributesMap.find(name);
  if (it != _attributesMap.end())
    return (*it).second.memberFunctor;
  
  return 0;    
}

X3DFieldLoader *SFType::getFieldLoader(const SFString &name) const
{
  map<SFString, FieldManager>::const_iterator it = _attributesMap.find(name);
  if (it != _attributesMap.end())
    return (*it).second.loader;
  
  return 0;
}

X3DFieldWriter *SFType::getFieldWriter(const SFString &name) const
{
  map<SFString, FieldManager>::const_iterator it = _attributesMap.find(name);
  if (it != _attributesMap.end())
    return (*it).second.writer;
  
  return 0;
}

void SFType::addVisitor(Visitor *visitor)
{
  //updating proxy list
  _actorList.push_back(visitor);
  
  //adding all SFTypes
  for (MFType::const_iterator it = _typeList.begin(); it != _typeList.end(); ++it)
    visitor->addType((*it).second);
}

void SFType::removeVisitor(Visitor *visitor)
{
  _actorList.remove(visitor);
}

void SFType::printInheritanceTree(SFString tab) const
{
  cout << tab << _component->getSceneGraph()->getName() << "::" << _name << ": " << _refCount << ", " << _component->getName() << ": " << _component << ", "  << ": " << _component->_refCount << endl;
  tab += "  ";  

  for (MFType::const_iterator it = _childList.begin(); it != _childList.end(); ++it)
    (*it).second->printInheritanceTree(tab);
}

SFString SFType::getEventIn(const SFString &name) const
{
  if (name.size() > 4)
  {
    SFString begin = name.substr(0, 4);
    if (begin == "set_")
      return name.substr(4, name.size());
  }
    
  return "";
}

SFString SFType::getEventOut(const SFString &name) const
{
  if (name.size() > 8)
  {
    SFString end = name.substr(name.size() - 8, 8);
    if (end == "_changed")
      return name.substr(0, name.size() - 8);
  }    
  
  return "";
}
  
