#include "X3DField.h"

namespace X3DTK {

const X3DField X3DField::null = X3DField(typeid(void), 0); 

X3DField::X3DField(const std::type_info &type, unsigned char *data)
: _type(type), _data(data)
{
}

void FieldLoader<bool>::load(X3DField &field, const SFString &value)
{
  *reinterpret_cast<bool *>(field.data()) = (value.upper() == "TRUE");
}

void FieldLoader<int>::load(X3DField &field, const SFString &value)
{
  *reinterpret_cast<int *>(field.data()) = value.toInt();
}

void FieldLoader<float>::load(X3DField &field, const SFString &value)
{
  *reinterpret_cast<float *>(field.data()) = value.toFloat();
}


SFString FieldWriter<bool>::write(const X3DField &field)
{
  if (*reinterpret_cast<bool *>(field.data()))
    return "TRUE";
  return "FALSE";  
}

SFString FieldWriter<int>::write(const X3DField &field)
{
  return SFString::number(*reinterpret_cast<int *>(field.data()));
}

SFString FieldWriter<float>::write(const X3DField &field)
{
  return SFString::number(*reinterpret_cast<float *>(field.data()));
}


void FieldWriter<bool>::writeToFile(SFString &output, const SFString &name, const X3DField &field, const SFString &init)
{
  SFString value;
  if (*reinterpret_cast<bool *>(field.data()))
    value = "TRUE";
  else
    value = "FALSE";  
    
  if (value != init)
    output += " " + name + "=\"" + value + "\"";
}

void FieldWriter<int>::writeToFile(SFString &output, const SFString &name, const X3DField &field, const SFString &init)
{
  SFString value = SFString::number(*reinterpret_cast<int *>(field.data()));
  if (value != init)
    output += " " + name + "=\"" + value + "\"";
}

void FieldWriter<float>::writeToFile(SFString &output, const SFString &name, const X3DField &field, const SFString &init)
{
  SFString value = SFString::number(*reinterpret_cast<float *>(field.data()));
  if (value != init)
    output += " " + name + "=\"" + value + "\"";
}


void FieldWriter<bool>::write(SFString &output, const bool &value)
{
  if (value)
    output += "TRUE";
  else  
    output += "FALSE";
}

void FieldWriter<int>::write(SFString &output, const int &value)
{
  output += SFString::number(value);
}

void FieldWriter<float>::write(SFString &output, const float &value)
{
  output += SFString::number(value);
}


}
